//******************************************************************************************
//* Inefficient Flame Fractal, runtime: 47 seconds
//******************************************************************************************

camera{orthographic location <0.45,-0.3,-3.75> look_at <0.45,-0.3,0> right x*image_width/image_height up y angle 52}

#declare rd=seed(1);
#macro Sqr(X) X*X #end
#macro Matrix(a,b,c,d) array[2][2]{{a,b},{c,d}} #end
#macro Dot(A,V) <A[0][0]*V.x+A[0][1]*V.y,A[1][0]*V.x+A[1][1]*V.y> #end

#macro r(p) vlength(p) #end
#macro theta(p) atan2(p.y,p.x) #end
#macro V0(p) p #end // linear
#macro V1(p) <sin(p.x),sin(p.y)> #end // sinusoidal
#macro V2(p) p/Sqr(r(p)) #end // spherical
#macro V3(p) #local r1=r(p); #local theta1=theta(p); r1*<cos(theta1+r1),sin(theta1+r1)> #end // swirl
#macro V4(p) #local theta1=theta(p); r(p)*<cos(2*theta1),sin(2*theta1)> #end // horseshoe
#macro V5(p) <theta(p)/pi,r(p)-1> #end // polar
#macro V6(p) #local r1=r(p); #local theta1=theta(p); r1*<sin(theta1+r1),sin(theta1-r1)> #end // handkerchief
#macro V7(p) #local r1=r(p); #local theta1=theta(p); r1*<sin(theta1*r1),-cos(theta1*r1)> #end // heart
#macro V8(p) #local r1=r(p); theta(p)*<sin(pi*r1),cos(pi*r1)> #end // disk
#macro V9(p) <p.x+p.y,p.y-p.x>/Sqr(r(p)) #end // spiral
#macro V10(p) <p.y,p.x>/Sqr(r(p)) #end // hyperbolic
#macro V11(p) #local r1=r(p); r1*<p.y*cos(r1),p.x*sin(r1)>/r1 #end // diamond
#macro V12(p) #local r1=r(p); #local theta1=theta(p); r1*<pow(sin(theta1+r1),3),pow(cos(theta1-r1),3)> #end // ex
#macro V13(p) #local omega1=theta(p)/2+(rand(rd)>0.5)*pi; sqrt(r1)*<cos(omega1),sin(omega1)> #end // julia

#macro F1(p) #local p2=Dot(Matrix(0.81,-0.33,0.08, 0.89),p)+<0.24,-0.07>; 0.88*V0(p2)+0.12*V1(p2) #end
#macro F2(p) #local p2=Dot(Matrix(0.3,0.52,-0.56,0.37),p) +<-0.09,-0.42>; 0.88*V0(p2)+0.12*V1(p2) #end
#macro F3(p) V1(Dot(Matrix(-0.43,0.38,-0.2,-0.44),p)+<1.74,1.21>) #end
#macro F4(p) V1(Dot(Matrix(-0.49,-0.27,0.28,-0.53),p)+<0.51,0.62>) #end

#declare w1=0.65; #declare w2=0.29; #declare w3=0.03; #declare w4=0.03;

#declare p=<0,0>; #declare i=0;
#while(i<10000) #declare X=rand(rd);
 #if(X<w1) #declare k=1; #else #if(X<w1+w2) #declare k=2; #else #if(X<w1+w2+w3) #declare k=3; #else #declare k=4; #end #end #end
 #switch(k)
  #case(1) #declare p=F1(p); #break
  #case(2) #declare p=F2(p); #break
  #case(3) #declare p=F3(p); #break
  #case(4) #declare p=F4(p); #break
 #end
 #declare hue=(k+3)/4;
 sphere{<p.x,p.y,0>,0.0075
  pigment{function{hue} color_map{[0 rgb <1,0,0>][1/6 rgb <1,1,0>][1/3 rgb <0,1,0>][1/2 rgb <0,1,1>][2/3 rgb <0,0,1>][5/6 rgb <1,0,1>][1 rgb <1,0,0>]}}
  finish{ambient 1}
 }
 #declare i=i+1;
#end
